/*
 * ll_testbed.cpp
 *
 *  Created on: Oct 28, 2019
 *      Author: abody
 */
#include "f4ll/console_handler.h"
#include "f4ll/crc_handler.h"
#include "f4ll/irq_lock.h"
#include "f4ll/memcpy_dma.h"
#include "f4ll/packet_usart.h"
#include "f4ll/str_util.h"
#include <stdlib.h>
#include <string.h>

extern "C" {
	#include "main.h"
	#include "globals.h"
	#include "config.h"
}
#include "globals.h"

#define PACKAGE_DELAY_MS 0
#define STATS_DELAY_MS 1000

#define ADDINFO(b,s,u) \
	b += strcpy_ex(b,s); \
	b += uitodec(b,u);

void PrintStats(f4ll::console_handler &con, uint8_t id, f4ll::packet_usart &usart)
{
    // char ids[] = " : ";
    // char *buffer = m_buffer;
    // packet_usart::stats const &stats = usart.get_stats();

    // ids[0] = id + '0';
    // buffer += strcpy_ex(buffer, ids);
    // ADDINFO(buffer, " s: ", stats.sent);
    // ADDINFO(buffer, " r: ", stats.rcvd);
    // ADDINFO(buffer, " sk: ", stats.skiped);
    // ADDINFO(buffer, " or: ", stats.overrun);
    // ADDINFO(buffer, " he: ", stats.hdr_error);
    // ADDINFO(buffer, " pe: ", stats.payload_errror);
    // buffer += strcpy_ex(buffer, ",0x");
    // buffer += uitohex(buffer, stats.pep1, 8);
    // buffer += strcpy_ex(buffer, ",0x");
    // buffer += uitohex(buffer, stats.pep2, 8);
    // ADDINFO(buffer, " rde: ", stats.rx_dma_error);
    // ADDINFO(buffer, " tde: ", stats.tx_dma_error);
    // ADDINFO(buffer, " pmh: ", stats.premature_hdr);
    // ADDINFO(buffer, " pmp: ", stats.premature_payload);
    // buffer += strcpy_ex(buffer, "\r\n");

    // setup_transmit(m_buffer, buffer - m_buffer + 1);
}

extern "C" void MainLoop()
{
	uint8_t const text2Send[]  __attribute__((aligned(16))) =
			"Megszentsegtelenithetetlensegeskedeseitekert\r\n"
			"--------------------------------------------\r\n\0\0\0";


	f4ll::memcpy_dma::init(MEMCPY_DMA_ENGINE, MEMCPY_DMA_STREAM);
    f4ll::crc_handler::init(DMA2, LL_DMA_STREAM_4);
    f4ll::console_handler::init(UART4, CONSOLE_DMA_ENGINE, 0u, CONSOLE_TX_DMA_STREAM);

	f4ll::packet_usart u1{ USART1, DMA2, LL_DMA_STREAM_2, LL_DMA_STREAM_7 };
	f4ll::packet_usart u2{ USART2, DMA1, LL_DMA_STREAM_5, LL_DMA_STREAM_6 };
	f4ll::packet_usart u3{ USART3, DMA1, LL_DMA_STREAM_1, LL_DMA_STREAM_3 };
	f4ll::packet_usart u6{ USART6, DMA2, LL_DMA_STREAM_1, LL_DMA_STREAM_6 };

	f4ll::packet_usart * usarts[] = { &u1, &u2, &u3, &u6 };
	f4ll::packet_usart **dst = g_usarts;
	for(auto usart: usarts)
		*dst++ = usart;

	uint32_t lastStatsTick = 0;
	uint32_t prevSentTick = 0;
	uint8_t	statId = 0;

	uint32_t	tmp = sizeof(text2Send) - 1;
	uint32_t	randmask = 0x80000000;

	do {
		if(randmask & tmp)
			break;
	} while((randmask = randmask >> 1));
	--randmask;

	lastStatsTick = HAL_GetTick();

	for(auto u : g_usarts)
		u->setup_receive();

	for(;;) {
		uint32_t tick = HAL_GetTick();
		bool send = PACKAGE_DELAY_MS ? (tick - prevSentTick > PACKAGE_DELAY_MS) : 1;

		if(send)
			prevSentTick += PACKAGE_DELAY_MS;

		for(auto u : g_usarts) {
			if(!u->is_tx_busy() && send) {
				//DIAG_ENTER_BUSY();
				auto len = sizeof(text2Send) - 1 - (rand() & randmask);
				f4ll::memcpy_dma::instance().copy(u->get_tx_packet_buffer(), text2Send, len);
				u->post_packet(nullptr, len);
				//DIAG_EXIT_BUSY();
			}
			for(uint16_t rIdx = 0; rIdx < 2; ++rIdx)
				if(u->is_rx_busy((bool)rIdx) || u->is_rx_failed(rIdx)) {
					u->get_rx_packet_buffer(rIdx);
					// ...
					u->rx_processed((bool)rIdx);
				}
		}
		if(tick - lastStatsTick > STATS_DELAY_MS) {
			PrintStats(f4ll::console_handler::instance(), statId, *g_usarts[statId]);
			lastStatsTick += STATS_DELAY_MS;
			++statId;
			if(statId >= sizeof(g_usarts) / sizeof(g_usarts[0]))
				statId = 0;
		}
//		uint32_t ein = LL_GPIO_ReadInputPort(KEY1_GPIO_Port);
//		if(!(ein & KEY1_Pin)) {
//			void (*fptr)(void) = (void (*)(void))(void*)0xa0000000;
//			fptr();
//		}
	}
}
